MODULE usrdef_istate
   !!======================================================================
   !!                   ***  MODULE  usrdef_istate   ***
   !!
   !!          ===  NORTH ATALNTIC ORCA025 IDEALISED OVERFLOWS  ===
   !!
   !! User defined : set the initial state of a user configuration
   !!======================================================================
   !! History :  4.0    ! 2016-03  (S. Flavoni) Original code
   !!            4.0.4  ! 2021-07  (D. Bruciaferri) Overflows code
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!  usr_def_istate : initial state in Temperature and salinity
   !!----------------------------------------------------------------------
   USE par_oce        ! ocean space and time domain
   USE phycst         ! physical constants
   USE dom_oce
   USE dtatsd     
   !
   USE in_out_manager ! I/O manager
   USE lib_mpp        ! MPP library
   
   IMPLICIT NONE
   PRIVATE

   PUBLIC   usr_def_istate   ! called in istate.F90

   !!----------------------------------------------------------------------
   !! NEMO/OCE 4.0 , NEMO Consortium (2018)
   !! $Id$
   !! Software governed by the CeCILL license (see ./LICENSE)
   !!----------------------------------------------------------------------
CONTAINS
  
   SUBROUTINE usr_def_istate( pdept, ptmask, pts, pu, pv, pssh )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE usr_def_istate  ***
      !! 
      !! ** Purpose :   Initialization of the dynamics and tracers
      !!                North Atlantic idealised overflows 
      !!
      !! ** Method  : - set temprature field
      !!              - set salinity   field
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk)     , INTENT(in   ) ::   pdept   ! depth of t-point               [m]
      REAL(wp), DIMENSION(jpi,jpj,jpk)     , INTENT(in   ) ::   ptmask  ! t-point ocean mask             [m]
      REAL(wp), DIMENSION(jpi,jpj,jpk,jpts), INTENT(  out) ::   pts     ! T & S fields      [Celsius ; g/kg]
      REAL(wp), DIMENSION(jpi,jpj,jpk)     , INTENT(  out) ::   pu      ! i-component of the velocity  [m/s] 
      REAL(wp), DIMENSION(jpi,jpj,jpk)     , INTENT(  out) ::   pv      ! j-component of the velocity  [m/s] 
      REAL(wp), DIMENSION(jpi,jpj)         , INTENT(  out) ::   pssh    ! sea-surface height
      !
      INTEGER                                              ::   ji, jj, kk, num_pnt, ierr0, ierr1
      INTEGER, ALLOCATABLE                 , DIMENSION(:)  ::   ovf_ji, ovf_jj ! arrays for overflow
      !!----------------------------------------------------------------------
      !
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'usr_def_istate : analytical definition of initial state '
      IF(lwp) WRITE(numout,*) '~~~~~~~~~~~~~~   Ocean at rest, with a 3D uniform T and S'
      IF(lwp) WRITE(numout,*) '                 and cold dense blobs in the DS and/or IFR'
      !
      pu  (:,:,:) = 0._wp        ! ocean at rest
      pv  (:,:,:) = 0._wp
      pssh(:,:)   = 0._wp
      !
      ! 3D uniform T & S profiles
      pts(:, :, :, jp_tem) = rn_tem_env
      pts(:, :, :, jp_sal) = rn_sal_env
      !
      SELECT CASE(nn_ovf_loc)

         CASE(0) ! Denmark Strait
            num_pnt = 17
            ALLOCATE( ovf_ji(num_pnt), STAT=ierr0 )
            ALLOCATE( ovf_jj(num_pnt), STAT=ierr1 )
            IF( ierr0 + ierr1 > 0 ) THEN
               CALL ctl_stop( 'dta_tsd : unable to allocate T & S data arrays' );   RETURN
            ENDIF
            ovf_ji = (/ &
                        &       1045, 1046, &
                        & 1044, 1045, 1046, &
                        & 1043, 1044, 1045, &
                        & 1042, 1043, 1044, &
                        & 1041, 1042, 1043, &
                        & 1040, 1041, 1042  &
                        & /)
            ovf_jj = (/ &
                        &       1020, 1020, &
                        & 1019, 1019, 1019, &
                        & 1018, 1018, 1018, &
                        & 1017, 1017, 1017, &
                        & 1016, 1016, 1016, &
                        & 1015, 1015, 1015  &
                        & /)
      END SELECT
      !
      IF( lk_mpp ) CALL mppsync
      DO kk = 1, num_pnt
         ji = ovf_ji(kk)
         jj = ovf_jj(kk)
         IF ((mi0(ji)>1 .AND. mi0(ji)<jpi) .AND. (mj0(jj)>1 .AND. mj0(jj)<jpj)) THEN
            pts(mi0(ji), mj0(jj), :, jp_tem) = rn_tem_ovf
            pts(mi0(ji), mj0(jj), :, jp_sal) = rn_sal_ovf
         ENDIF
      END DO           
      !   
   END SUBROUTINE usr_def_istate

   !!======================================================================
END MODULE usrdef_istate
